
class RangeControl:
    def __init__(self, *args):
        """ Constructs a type agnostic range control, may be called either 
            with all of these parameters set or without any (then it 
            defaults to int type: 0, 99, 1, 10, 0)
            RangeControl(minValue, maxValue, lineStep, pageStep, value)
            Decimal is good alternative type, float isn't..
        """
        self.minVal = 0
        self.maxVal = 99
        self.lineStep = 1
        self.pageStep = 10
        self.val = 0
        self.prevVal = -1

        if args:
            if len(args) == 5:
                val = args[4]
                self.minVal = args[0]
                self.maxVal = args[1]
                self.lineStep = abs(args[2])
                self.pageStep = abs(args[3])
                self.val = self.bound(val)
                self.prevVal = self.minVal - 1
            else:
                raise "RangeControl: wrong number of arguments: 5 expected"

    def value(self):
        return self.val

    def prevValue(self):
        return self.prevVal

    def minValue(self):
        return self.minVal

    def maxValue(self):
        return self.maxVal

    def lineStep(self):
        return self.lineStep

    def pageStep(self):
        return self.pageStep

    def setValue(self, value):
        self.directSetValue(value)
        if self.prevVal != self.val:
            self.valueChange()

    def directSetValue(self, value):
        self.prevVal = self.val
        self.val = self.bound(value)

    def addPage(self):
        off = min(self.maxVal - self.val, self.pageStep)
        if off:
            self.setValue(self.val + off)

    def subtractPage(self):
        off = min(self.val - self.minVal, self.pageStep)
        if off:
            self.setValue(self.val - off)

    def addLine(self):
        off = min(self.maxVal - self.val, self.lineStep)
        if off:
            self.setValue(self.val + off)

    def subtractLine(self):
        off = min(self.val - self.minVal, self.lineStep)
        if off:
            self.setValue(self.val - off)

    def setMinValue(self, minVal):
        maxVal = self.maxVal
        if maxVal < minVal:
            maxVal = minVal
        self.setRange(minVal, maxVal)

    def setMaxValue(self, maxVal):
        minVal = self.minVal
        if minVal > maxVal:
            minVal = maxVal
        self.setRange(minVal, maxVal)

    def setRange(self, minValue, maxValue):
        if minValue > maxValue:
            print "RangeControl.setRange: minValue %s > maxValue %s" % (
                    minValue, maxValue)
            maxValue = minValue

        if minValue == self.minVal and maxValue == self.maxVal:
            return
        self.minVal = minValue
        self.maxVal = maxValue
        tmp = self.bound(self.val)
        self.rangeChange()
        if tmp != self.val:
            self.prevVal = val
            self.val = tmp
            self.valueChange()

    def setSteps(self, lineStep, pageStep):
        if lineStep != self.lineStep or pageStep != self.pageStep:
            self.lineStep = abs(lineStep)
            self.pageStep = abs(pageStep)
            self.stepChange()

    def valueChange(self):
        pass

    def rangeChange(self):
        pass

    def stepChange(self):
        pass

    def bound(self, v):
        if v < self.minVal:
            return self.minVal
        if v > self.maxVal:
            return self.maxVal
        return v
